#include <Servo.h>

int PIN_EL_SERVO = 6;
int PIN_AZ_SERVO = 5;
int PIN_BLED = 4;
int PIN_LASER = 3;
int PIN_RLED = 2;

int AZIMUTH = 0;
int ELEVATION = 90;

double a_a = 0.276;
double b_a = 40.276;
double a_e = 0.87;
double b_e = 11.7;

int LED_MODE_OFF       = 0;
int LED_MODE_ON        = 1;
int LED_MODE_BLINK     = 2;

int LASER_MODE_OFF     = 0;
int LASER_MODE_ON      = 1;

int BLED_MODE = LED_MODE_OFF;
int RLED_MODE = LED_MODE_OFF;
int LASER_MODE = LASER_MODE_OFF;

Servo elServo;
Servo azServo;
boolean AZ_SERVO_CLOCKWISE = false;

int req[20]={};
int reqLength;
int res[20]={};
int resLength;
int incomingByte;

void setup() {
  pinMode(PIN_BLED, OUTPUT);
  pinMode(PIN_LASER, OUTPUT);
  pinMode(PIN_RLED, OUTPUT);
  onoff();
  elServo.attach(PIN_EL_SERVO);
  azServo.attach(PIN_AZ_SERVO);
  turn(AZIMUTH,ELEVATION,true);
  Serial.begin(115200);
}

void loop() {
  reqLength = Serial.available();
  if (reqLength > 0) {
    //Serial.print("%loop:reqLength=");Serial.println(reqLength);
    for (int i=0; i < reqLength; i++) {
      incomingByte = Serial.read();
      req[i] = incomingByte;
      //Serial.print("%loop:req[");Serial.print(i);Serial.print("]=");Serial.println(req[i]);
    }
    exec();
    Serial.print("OK$");
  }
  delay(300);
}
void exec() {
  Serial.println("%exec:START");
  if(req[0]=='S'&&reqLength>3){
    BLED_MODE = req[1]-'0';
    LASER_MODE = req[2]-'0';
    RLED_MODE = req[3]-'0';
    onoff();
    Serial.print("%exec:BLED_MODE=");Serial.print(BLED_MODE);Serial.print(",LASER_MODE=");Serial.print(LASER_MODE);Serial.print(",RLED_MODE=");Serial.println(RLED_MODE);
  }else if(req[0]=='R'){
    if(reqLength>5){
      AZIMUTH = (req[1]-'0')*100+(req[2]-'0')*10+int(req[3]-'0');
      ELEVATION = (req[4]-'0')*10+(req[5]-'0');
    }else if(reqLength>3){
      AZIMUTH = (req[1]-'0')*100+(req[2]-'0')*10+int(req[3]-'0');
    }else if(reqLength>2){
      ELEVATION = (req[1]-'0')*10+(req[2]-'0');
    }
    turn(AZIMUTH,ELEVATION,true);
    Serial.print("%exec:AZIMUTH=");Serial.print(AZIMUTH);Serial.print(",ELEVATION=");Serial.println(ELEVATION);
  }else if(req[0]=='r'&&reqLength>7){
    AZIMUTH = (req[1]-'0')*100+(req[2]-'0')*10+int(req[3]-'0');
    ELEVATION = (req[4]-'0')*100+(req[5]-'0')*10+int(req[6]-'0');
    turn(AZIMUTH,ELEVATION,false);
    Serial.print("%exec:AZIMUTH(R)=");Serial.print(AZIMUTH);Serial.print(",ELEVATION(R)=");Serial.println(ELEVATION);
  }else if(req[0]=='a'&&reqLength>8){
    double tmp = (req[3]-'0')*10+(req[4]-'0')+int(req[5]-'0')/10.0+int(req[6]-'0')/100.0+int(req[7]-'0')/1000.0;
    if(req[2]=='-')tmp = -tmp;
    if(req[1]=='a'){
      a_a = tmp;
      Serial.print("%exec:a_a=");Serial.println(tmp);
    }else if(req[1]=='e'){
      a_e = tmp;
      Serial.print("%exec:a_e=");Serial.println(tmp);
    }
    turn(AZIMUTH,ELEVATION,true);
  }else if(req[0]=='b'&&reqLength>8){
    double tmp = (req[3]-'0')*10+(req[4]-'0')+int(req[5]-'0')/10.0+int(req[6]-'0')/100.0+int(req[7]-'0')/1000.0;
    if(req[2]=='-')tmp = -tmp;
    if(req[1]=='a'){
      b_a = tmp;
      Serial.print("%exec:b_a=");Serial.println(tmp);
    }else if(req[1]=='e'){
      b_e = tmp;
      Serial.print("%exec:b_e=");Serial.println(tmp);
    }
    turn(AZIMUTH,ELEVATION,true);
  }
  req[0] = 'D';  //DONE
  Serial.println("%exec:END");
}
int transAz(int virtualAz) {
  return a_a*virtualAz+b_a;
}
int transEl(int virtualEl) {
  return a_e*virtualEl+b_e;
}
void onoff() {
  if(BLED_MODE==LED_MODE_ON&&digitalRead(PIN_BLED)==LOW){
    digitalWrite(PIN_BLED,HIGH);
  }else if(BLED_MODE==LED_MODE_OFF&&digitalRead(PIN_BLED)==HIGH){
    digitalWrite(PIN_BLED,LOW);
  }
  if(LASER_MODE==LASER_MODE_ON&&digitalRead(PIN_LASER)==LOW){
    digitalWrite(PIN_LASER,HIGH);
  }else if(LASER_MODE==LASER_MODE_OFF&&digitalRead(PIN_LASER)==HIGH){
    digitalWrite(PIN_LASER,LOW);
  }
  if(RLED_MODE==LED_MODE_ON&&digitalRead(PIN_RLED)==LOW){
    digitalWrite(PIN_RLED,HIGH);
  }else if(RLED_MODE==LED_MODE_OFF&&digitalRead(PIN_RLED)==HIGH){
    digitalWrite(PIN_RLED,LOW);
  }
}
int lastAz = -1;
int lastEl = -1;
void turn(int az,int el,boolean trans) {
  if(trans){
    if(!AZ_SERVO_CLOCKWISE){
      az = 360 - az;
    }
    az = transAz(az);
    el = transEl(el);
  }else if(!AZ_SERVO_CLOCKWISE){
    az = 180 - az;
  }
  int azDiff = lastAz - az;
  if(azDiff<0)azDiff = -azDiff;
  int turnoff = 1;
  if(lastAz>0&&azDiff<10){
    turnoff = 0;
  }
  if(turnoff>0){
    digitalWrite(PIN_BLED,LOW);
    digitalWrite(PIN_LASER,LOW);
    digitalWrite(PIN_RLED,LOW);
  }
  elServo.write(el);
  azServo.write(az);
  if(turnoff>0){
    delay(2000);
    onoff();
  }
  lastEl = el;
  lastAz = az;
}
